{
        Program Name : AutoUpdate
        Package Name : Application Foundation
        Purpose      : Check for web updates.
        Institution  : Conroy & Conroy Co.
        Date Written : 16-Dec-2002
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 2002 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          This program checks for web updates by downloading a file from a
        URL and executing the commands therein.  Each line of the download file
        must be of the form: <target>=<source>
        where "<source>" is the URL of a file to download (HTTP only) and
        "<target>" is where to place the file on the user's machine.  One
        special line is simply the text "EXECUTE".  The following line is the
        name of a file to execute.  Typically, the first line indicates an
        executable to download (some install program or some such), followed by
        the execute command to execute that program.
          The program exits silently if it cannot access the specified URL.  It
        also exits as soon as any line the download file cannot be executed for
        any reason.

          Command line: <url> <path> {<translations>} {/NOASK}
{
          where <url> is the url of the download file.
          <path> is name and path of the cached version of the download file.
          If the file at <url> is the same as the one at <path>, the program
          exits silently (since there is no work to do).
          <translations> is a file consisting of string substitutions to perform
          on the commands in the download file.  This allows the files to be
          placed in the proper directory on the system where the program is
          running.  The translation file is optional.  If not provided, no
          substitutions are done to the text in the download file.  The
          translation file is a text file where each line indicates a single
          substitution.  Each line has the format: <string>=<substitution>  For
          each line in the download file, each matching string from the
          translation file is replaced by the substitution string before the
          file copy or execution takes place.  Note that strings or
          substitution strings must be enclosed in quotes if they contain equal
          signs (=).  For instance, if a line in the download file was:
              <program>\temp=http://www.mysite.com/NewProg.exe
          and the translation file had a line:
              <program>="c:\Program Files\MyProg\"
          then the file at http://www.mysite.com/NewProg.exe would be copied to
          c:\Program Files\MyProg\temp\NewProg.exe.

            Optionally, the /ASK switch can be used to specify that the user is
          to be prompted as to whether or not to do the download.  The /ASK
          switch may have an optional query string following it which is the
          text of the query shown to the user.  If omitted, the user is asked if
          they want to download from the command-line URL value.  /NOASK can be
          used to suppress the user query.

            Note that the command line parameters must be delimited by quotes if
          the parameter contains a space or slash character.  Either the single
          quote (apostrophe - ') or double quote (") can be used for this
          purpose.  This also applies to the parameter of the ASK switch.  Here
          are example command lines:

          "http://www.mysite.com/updates.txt" "c:\Program Files\MyProg\updates.txt" "c:\Program Files\MyProg\xlate.txt" /ASK="There is a new version of MyProg available.  Do you want to download it?"
          "http://www.mysite.com/updates.txt" 'c:\Program Files\MyProg\updates.txt' /ASK
          "http://www.mysite.com/updates.txt" c:\myprog.txt /NOASK

          Example download file contents:

          <program>Setup.exe=http://www.mysite.com/Setup.exe
          execute <program>Setup.exe

          This will download the file from http://www.mysite.com/Setup.exe to
          the current system (exact location depends upon the contents of the
          translation file).  Then it executes the local copy of the file.  This
          is the typical means of providing updates - a single download of an
          installation program.
}

program AutoUpdate ;

uses // Borland...
     Windows, // WinExec
     Dialogs, // MessageBox
     Classes, // TStringList
     SysUtils, // uppercase

     // C&C...
     Internet_Util, // Get_Web_File
     Parse, // Parse_Switch
     Standard, // ERT
     UStrings ; // Trim_Quotes

var Translations : TStringList = nil ;

function Translate_Target( Target : string ) : string ;

var Dummy, Loop : integer ;
    T, Work : string ;
    _Target : string ;

begin
    _Target := uppercase( Target ) ;
    for Loop := 0 to Translations.Count - 1 do
    begin
        Work := Translations[ Loop ] ;
        T := Parse_Parameter( '=', Work ) ;
        Dummy := pos( uppercase( T ), _Target ) ;
        if( Dummy > 0 ) then
        begin
            Target := copy( Target, 1, Dummy - 1 ) + Work + copy( Target, Dummy + 1, length( Target ) ) ;
            _Target := uppercase( Target ) ;
        end ;
    end ;
    Result := Target ;
end ;


var Ask : boolean ;
    Command : string ;
    Dummy : integer ;
    F : file ;
    Path : string ; // Path and name of where to cache update file
    Target : string ;
    URL : string ; // URL of update file

    function Copy_File( Source, Target : string ) : boolean ;

    var Dummy : integer ;
        S : string ;

    begin
        Result := False ; // Assume failure

        // Retrieve source file...
        S := Get_Web_File( Source ) ;
        if( length( S ) = 0 ) then
        begin
            exit ;
        end ;

        // Open/create target file...
        assignfile( F, Target ) ;
        {$I-}
        rewrite( F, 1 ) ;
        {$I+}
        Dummy := IOResult ;
        if( Dummy <> 0 ) then
        begin
            ShowMessage( ERT( Dummy ) + ' while updating file ' + Target ) ;
            exit ;
        end ;
        {$I-}
        blockwrite( F, S[ 1 ], length( S ) ) ;
        {$I+}
        Dummy := IOResult ;

        // Close file...
        {$I-}
        closefile( F ) ;
        {$I+}
        IOResult ;
        if( Dummy <> 0 ) then
        begin
            ShowMessage( ERT( Dummy ) + ' while updating file ' + Target ) ;
        end else
        begin
            Result := True ; // Success
        end ;
    end ; // Copy_File


procedure Process_File( Name : string ) ;

var F : textfile ;
    S : string ;
    
begin
    // Process the file...
    assignfile( F, Name ) ;
    {$I-}
    reset( F ) ;
    {$I+}
    if( IOResult = 0 ) then
    begin
        while( not eof( F ) ) do
        begin
            {$I-}
            readln( F, S ) ;
            {$I+}
            if( IOResult <> 0 ) then
            begin
                break ;
            end ;
            if( uppercase( copy( S, 1, 8 ) ) = 'EXECUTE ' ) then
            begin
                Target := Parse_Parameter( ' ', S ) ;
                S := Translate_Target( S ) ;
                WinExec( PChar( S ), SW_NORMAL ) ;
                continue ;
            end ;
            Target := Translate_Target( Parse_Parameter( '=', S ) ) ;
            if( ( length( Target ) = 0 ) or ( length( S ) = 0 ) ) then // Invalid file format
            begin
                {$I-}
                closefile( F ) ;
                {$I+}
                IOResult ;
                exit ;
            end ;
            if( not Copy_File( S, Target ) ) then // Couldn't complete the update
            begin
                {$I-}
                closefile( F ) ;
                {$I+}
                IOResult ;
                exit ;
            end ;
        end ;
    end ;
end ;

var Ask_String : string ;
    FI : textfile ;
    S, S1 : string ;
    
begin
    // Command line format: Autoupdate <url> <path> {<translation file>} {/ASK=x}
    Command := Command_Line ;
    Ask := ( Parse_Switch( 'A|SK', 'NOA|SK', Command, Ask_String ) <> 2 ) ;
    if( Switch_Present( Command ) > 0 ) then
    begin
        ShowMessage( 'Unknown Autoupdate switch: ' + Command ) ;
        exit ; // All other switches are unknown
    end ;
    URL := Parse_Parameter( ' ', Command ) ;
    if( length( Ask_String ) = 0 ) then
    begin
        Ask_String := 'Do you want to update from ' + URL + '?' ;
    end ;
    Path := Parse_Parameter( ' ', Command ) ;
    Translations := TStringList.Create ;
    if( length( Command ) > 0 ) then // Translation file supplied
    begin
        Command := Trim_Quotes( Command ) ;
        assignfile( FI, Command ) ;
        {$I-}
        reset( FI ) ;
        {$I+}
        Dummy := IOResult ;
        if( Dummy <> 0 ) then
        begin
            ShowMessage( ERT( Dummy ) + ' while opening Autoupdate file ' + Command ) ;
            exit ;
        end ;
        while( not eof( FI ) ) do
        begin
            {$I-}
            readln( FI, S ) ;
            {$I+}
            Dummy := IOResult ;
            if( Dummy <> 0 ) then
            begin
                ShowMessage( ERT( Dummy ) + ' while reading Autoupdate file ' + Command ) ;
                {$I-}
                closefile( FI ) ;
                {$I+}
                IOResult ;
                exit ;
            end ;
            Translations.Add( S ) ;
        end ; // while( not eof( F ) )

        // Close the file and ignore errors...
        {$I-}
        closefile( F ) ;
        {$I+}
        IOResult ;
    end ; // if( length( Command ) > 0 )
    if( length( URL ) + length( Path ) > 0 ) then
    begin
        if( not Connected_To_Internet ) then
        begin
            exit ;
        end ;

        URL := Trim_Quotes( URL ) ;
        Path := Trim_Quotes( Path ) ;

        try
            // Read in the old version of the notice file...
            assignfile( F, Path ) ;
            {$I-}
            reset( F, 1 ) ;
            {$I+}
            if( IOResult = 0 ) then // We have a previous version
            begin
                setlength( S1, filesize( F ) ) ;
                {$I-}
                blockread( F, S1[ 1 ], length( S1 ) ) ;
                {$I+}
                IOResult ;
                {$I-}
                closefile( F ) ;
                {$I+}
                IOResult ;
            end else
            begin
                S1 := '' ;
            end ;

            S := Get_Web_File( URL ) ;
            if( length( S ) = 0 ) then // Not found
            begin
                exit ;
            end ;

            // See if there is a change...
            if( S = S1 ) then
            begin
                exit ; // No change
            end else
            begin
                if( Ask ) then
                begin
                    if( MessageBox( 0, PChar( Ask_String ), 'AutoUpdate', MB_YESNO ) = IDNO ) then
                    begin
                        {$I-}
                        closefile( F ) ;
                        {$I+}
                        IOResult ;
                        exit ;
                    end ;
                end ;
                {$I-}
                rewrite( F, 1 ) ;
                {$I+}
                if( IOResult = 0 ) then
                begin
                    {$I-}
                    blockwrite( F, S[ 1 ], length( S ) ) ;
                    {$I+}
                    IOResult ;
                end ;
            end ;

            // Close the file...
            {$I-}
            closefile( F ) ;
            {$I+}
            IOResult ;

            Process_File( Path ) ;
        except
            exit ; // Ignore exceptions
        end ;
    end ; // if( length( URL ) > 0 )
end.
